import Decimal from "../libs/decimal/decimal";
import { AStar } from "./aStar/AStar";
import { AStarGrid } from "./aStar/AStarGrid";
import { AES, enc, mode, pad } from "./crypto/crypto-ts";
import { Base64 } from "./crypto/enc/Base64";
import MD5_TS from "./md5/MD5_TS";
import pako from "../libs/pako/pako";

const { ccclass, property } = cc._decorator;


@ccclass
export default class MainScene extends cc.Component {

    onLoad() {
        //this.astarTest();
        //this.protoTest();
        //this.jszipTest();
        //this.decimalTest();
        //this.cryPtoTest();
        //this.md5Test();
        //this.socketIOTest();
        this.pakoTest();
    }


    /**A星寻路 */
    private astarTest() {
        //创建网格
        let grid: AStarGrid = new AStarGrid(20, 20);
        //设置不可行走点
        for (var i = 0; i < 100; i++) {
            grid.setWalkable(Math.floor(Math.random() * 20),
                Math.floor(Math.random() * 20),
                false);
        }
        //设置起点
        grid.setStartNode(0, 0);
        //设置终点
        grid.setEndNode(19, 19);
        //开始寻路
        let aStar: AStar = new AStar();
        if (aStar.findPath(grid)) {
            console.log("寻路成功，路径点:", aStar.path);
        } else {
            console.log("寻路失败");
        }
    }

    /**ProtoBuf */
    private protoTest() {
        let roomInfo: game.RoomInfo = new game.RoomInfo();
        roomInfo.roomId = 123;
        let sendData = game.RoomInfo.encode(roomInfo).finish();
        let revData = game.RoomInfo.decode(sendData);
        console.log(revData);
    }

    /**JsZip */
    private jszipTest() {
        //加载config.bin
        cc.resources.load("config_zip/GameItem", (err, assets: any) => {
            console.log("A:", assets);
            //解析
            JSZip.loadAsync(assets._buffer).then((zip) => {
                console.log("B:", zip);
                //获取GameItem.json配置
                zip.file("GameItem.json").async("text").then((data) => {
                    //string转成json格式
                    let json = JSON.parse(data);
                    console.log(json["name"]);
                })
            });
        })
    }

    /**Deciaml */
    private decimalTest() {
        let a = new Decimal(100);
        console.log(a, a.toString(), a.toNumber());
        let b = new Decimal(999999999999999 * 999999999999999);
        console.log(b, b.toString(), b.toNumber());
        //加法
        console.log(new Decimal(999).add(1).toNumber());
        //减法
        console.log(new Decimal(1000).sub(1).toNumber());
        //除法
        console.log(new Decimal(1000).div(2).toNumber());
        //乘法
        console.log(new Decimal(9).mul(9).toNumber());
        //是否相等
        console.log(new Decimal(9).equals(9));
        //是否大于
        console.log(new Decimal(9).gt(10));
    }

    /**Crypto测试 */
    private cryPtoTest() {
        //密钥
        let key = enc.Utf8.parse("1234567890123456");
        //密钥偏移量
        let iv = enc.Utf8.parse("abcdefabcdea1234");
        //待加密字符串
        let word = "123abc";
        //加密
        let srcs = enc.Utf8.parse(word);
        let encrypted = AES.encrypt(srcs, key, { iv: iv, mode: mode.CBC, padding: pad.PKCS7 });
        let encryptedStr = encrypted.ciphertext.toString().toUpperCase();
        console.log("加密后:", encryptedStr);
        //解密
        let encryptedHex = enc.Hex.parse(encryptedStr);
        let srcs2 = Base64.stringify(encryptedHex);
        let decrypt = AES.decrypt(srcs2, key, { iv: iv, mode: mode.CBC, padding: pad.PKCS7 });
        let decryptStr = decrypt.toString(enc.Utf8);
        console.log("解密:", decryptStr);
    }

    /**MD5测试 */
    private md5Test() {
        let str = new MD5_TS().hex_md5("123abc");
        console.log(str);
    }

    /**sockeIO测试 */
    private socketIOTest() {
        let socket;
        socket = io.connect("http://127.0.0.1:3004", { reconnection: false, 'force new connection': true });
        socket.on('connect', function () {
            console.log("connect success");
        });
    }

    /**pako测试 */
    private pakoTest() {
        console.log(this.toUint8Arr("阿阿阿阿阿哦哦哦哦哦11111aaaaa非非非非非非非非非非非非"));

        let a = pako.deflate("阿阿阿阿阿哦哦哦哦哦11111aaaaa非非非非非非非非非非非非", { level: 1 });
        console.log(a);
        let b = pako.inflate(a, { to: "string" });
        console.log(b);
        let c = pako.gzip("阿阿阿阿阿哦哦哦哦哦11111aaaaa非非非非非非非非非非非非");
        console.log(c);
        let d = pako.ungzip(c, { to: "string" });
        console.log(d);
    }

    /**字符串转Uint8Array */
    private toUint8Arr(str) {
        const buffer = [];
        for (let i of str) {
            const _code = i.charCodeAt(0);
            if (_code < 0x80) {
                buffer.push(_code);
            } else if (_code < 0x800) {
                buffer.push(0xc0 + (_code >> 6));
                buffer.push(0x80 + (_code & 0x3f));
            } else if (_code < 0x10000) {
                buffer.push(0xe0 + (_code >> 12));
                buffer.push(0x80 + (_code >> 6 & 0x3f));
                buffer.push(0x80 + (_code & 0x3f));
            }
        }
        return Uint8Array.from(buffer);
    }
}


declare let io: any;
declare interface Window {
    io: any
}

declare let pako: any;
declare interface Window {
    pako: any
}